#!/usr/bin/bash

# vim:ft=sh:fdm=marker:fmr={,}

# archlabs installer library script file
# this file is not meant to be run directly
# sourcing this file in a non bash shell is not advised

# set -n

format() {
    infobox "$_FSTitle" "\nFormatting: $1\n\nCommand: ${FS_CMDS[$2]}\n" 0
    ${FS_CMDS[$2]} $1 >/dev/null 2>$ERR
    echeck "${FS_CMDS[$2]} $1"
}

partition() {
    local device
    if [[ $# -eq 0 ]]; then
        select_device 'root' || return 1
        device="$DEVICE"
    else
        device="$1"
    fi

    tput civis
    local choice
    choice="$(dialog --cr-wrap --stdout --backtitle "$BT" --title " $_PartTitle " \
        --menu "$_PartBody" 0 0 0 "$_PartShowTree" "-" "$_PartAuto" "-" \
        $( ([[ $DISPLAY ]] && hash gparted >/dev/null 2>&1) && printf "gparted -") \
        "cfdisk" "-" "parted" "-" "$_PartWipe" "-")"
    [[ $? != 0 || $choice == "" ]] && return 1

    tput civis
    if [[ $choice != "$_PartWipe" && $choice != "$_PartAuto" && $choice != "$_PartShowTree" ]]; then
        clear; tput cnorm; $choice $device
    elif [[ $choice == "$_PartShowTree" ]]; then
        msgbox "$_PrepShowDev" "\n$(lsblk -o NAME,MODEL,TYPE,FSTYPE,SIZE,MOUNTPOINT "$device")\n"
        partition $device
    elif [[ $choice == "$_PartWipe" ]]; then
        yesno "$_PartWipe" "$_PartBody1 $device $_PartWipeBody2" && wipe -Ifrev $device
        partition $device
    else
        # if auto_partition fails we need to re-initialize the variables, just to be sure
        auto_partition $device || { init; return 1; }
    fi
}

decr_count() {
    # remove a partition from the dialog list and decrement the number partitions left
    (( $# == 1 )) || return 1

    local p="$1"
    PARTS="$(sed "s~${p} [0-9]*[G-M]~~; s~${p} [0-9]*\.[0-9]*[G-M]~~" <<< "$PARTS")"
    (( COUNT > 0 )) && (( COUNT-- ))
    return 0
}

enable_swap() {
    if [[ $1 == "$MNT/swapfile" ]]; then
        fallocate -l $SWAP_SIZE $1 2>$ERR
        echeck "fallocate -l $SWAP_SIZE $1"
        chmod 600 $1 2>$ERR
        echeck "chmod 600 $1"
    fi
    mkswap $1 >/dev/null 2>$ERR
    echeck "mkswap $1"
    swapon $1 >/dev/null 2>$ERR
    echeck "swapon $1"
    return 0
}

device_tree() {
    tput civis
    local msg
    if [[ $IGNORE_DEV != "" ]]; then
        msg="$(lsblk -o NAME,MODEL,TYPE,FSTYPE,SIZE,MOUNTPOINT |
            awk "!/$IGNORE_DEV/"' && /disk|part|lvm|crypt|NAME/ {print $0}')"
    else
        msg="$(lsblk -o NAME,MODEL,TYPE,FSTYPE,SIZE,MOUNTPOINT |
            awk '/disk|part|lvm|crypt|NAME/ {print $0}')"
    fi
    msgbox "$_PrepShowDev" "$msg"
}

select_device() {
    local dev
    local msg
    [[ $1 == 'boot' ]] && msg="$_DevSelTitle for bootloader\n" || unmountp

    if [[ $DEV_COUNT -eq 1 && $SYS_DEVS ]]; then
        DEVICE="$(awk '{print $1}' <<< "$SYS_DEVS")"
        msg="\nOnly one device available$([[ $1 == 'boot' ]] && printf " for grub bootloader"):"
        infobox "$_DevSelTitle" "$msg $DEVICE\n" 1
    elif (( DEV_COUNT > 1 )); then
        tput civis
        DEVICE="$(dialog --cr-wrap --stdout --backtitle "$BT" --title " $_DevSelTitle " \
            --menu "${msg}$_DevSelBody" 0 0 0 $SYS_DEVS)"
        [[ $? != 0 || $DEVICE == "" ]] && return 1
    else
        msg="\nNo available devices for installation to use$([[ $1 == 'boot' ]] && printf " for grub bootloader")."
        msgbox "$_ErrTitle" "$msg\n$_Exit"
        die 1
    fi

    # if the device selected was for grub bootloader, set the BOOT_DEVICE
    # this is needed because grub uses the base device for BIOS, not the partition
    [[ $1 == 'boot' ]] && BOOT_DEVICE="$DEVICE"
    return 0
}

confirm_mount() {
    local part="$1"
    local mount="$2"
    local msg="Partition: $part\n"
    [[ $mount == "$MNT" ]] && msg+="Mountpoint: / (root)" || msg+="Mountpoint: ${mount#$MNT}"

    if [[ $(mount) =~ "$mount" ]]; then
        infobox "$_MntTitle" "$_MntSucc\n$msg\n" 1 && decr_count "$part"
    else
        infobox "$_MntTitle" "$_MntFail\n$msg\n" 1 && return 1
    fi
    return 0
}

check_cryptlvm() {
    local dev=""
    local part="$1"
    local devs="$(lsblk -lno NAME,FSTYPE,TYPE)"

    # Identify if $part is "crypt" (LUKS on LVM, or LUKS alone)
    if [[ $(lsblk -lno TYPE "$part") =~ 'crypt' ]]; then
        LUKS=' encrypted'
        LUKS_NAME="${part#/dev/mapper/}"

        for dev in $(awk '/lvm/ && /crypto_LUKS/ {print "/dev/mapper/"$1}' <<< "$devs" | uniq); do
            if grep -q "$LUKS_NAME" <<< "$(lsblk -lno NAME "$dev")"; then
                LUKS_DEV="$LUKS_DEV cryptdevice=$dev:$LUKS_NAME"
                LVM=' logical volume'
                break
            fi
        done
        for dev in $(awk '/part/ && /crypto_LUKS/ {print "/dev/"$1}' <<< "$devs" | uniq); do
            if grep -q "$LUKS_NAME" <<< "$(lsblk -lno NAME "$dev")"; then
                LUKS_UUID="$(lsblk -lno UUID,TYPE,FSTYPE "$dev" | awk '/part/ && /crypto_LUKS/ {print $1}')"
                LUKS_DEV="$LUKS_DEV cryptdevice=UUID=$LUKS_UUID:$LUKS_NAME"
                break
            fi
        done

    elif [[ $(lsblk -lno TYPE "$part") =~ 'lvm' ]]; then
        LVM=' logical volume'
        VOLUME_NAME="${part#/dev/mapper/}"

        for dev in $(awk '/crypt/ && /lvm2_member/ {print "/dev/mapper/"$1}' <<< "$devs" | uniq); do
            if grep -q "$VOLUME_NAME" <<< "$(lsblk -lno NAME "$dev")"; then
                LUKS_NAME="$(sed 's~/dev/mapper/~~g' <<< "$dev")"
                break
            fi
        done
        for dev in $(awk '/part/ && /crypto_LUKS/ {print "/dev/"$1}' <<< "$devs" | uniq); do
            if grep -q "$LUKS_NAME" <<< "$(lsblk -lno NAME "$dev")"; then
                LUKS_UUID="$(lsblk -lno UUID,TYPE,FSTYPE "$dev" | awk '/part/ && /crypto_LUKS/ {print $1}')"
                LUKS_DEV="$LUKS_DEV cryptdevice=UUID=$LUKS_UUID:$LUKS_NAME"
                LUKS=' encrypted'
                break
            fi
        done
    fi
}

auto_partition() {
    local device="$1"
    local size=$(lsblk -lno SIZE $device | awk 'NR == 1 {
    if ($1 ~ "G") {
        sub(/G/, ""); print ($1 * 1000 - 512) / 1000"G"
    } else {
        sub(/M/, ""); print ($1 - 512)"M"
    }}')

    # confirm or bail
    yesno "$_PrepParts" "$_PartBody1 $device $_PartBody2 ($size)$_PartBody3" || return 0

    local newtable format
    if [[ $SYS == BIOS ]]; then
        newtable="msdos"; format="ext4"
    else
        newtable="gpt"; format="fat32"
    fi

    infobox "$_PrepParts" "\nRemoving existing partitions on $device and setting partition table to $newtable\n" 0

    swapoff -a  # in case the device was previously used for swap
    local dev_info="$(parted -s $device print)"

    # walk the partitions on the device in reverse order and delete them
    for i in $(awk '/^ [1-9][0-9]?/ {print $1}' <<< "$dev_info" | sort -r); do
        parted -s $device rm $i >/dev/null 2>&1
    done

    if [[ $(awk '/Table:/ {print $3}' <<< "$dev_info") != "$newtable" ]]; then
        parted -s $device mklabel $newtable >/dev/null 2>&1
    fi

    infobox "$_PrepParts" "\nCreating a 512M $format boot partition.\n" 0
    if [[ $SYS == "BIOS" ]]; then
        parted -s $device mkpart primary $format 1MiB 513MiB >/dev/null 2>&1
    else
        parted -s $device mkpart ESP $format 1MiB 513MiB >/dev/null 2>&1
    fi

    sleep 0.1
    BOOT_DEVICE="$device"
    BOOT_PART=$(lsblk -lno NAME,TYPE $device | awk 'NR == 2 {print "/dev/"$1}')

    if [[ $SYS == "BIOS" ]]; then
        mkfs.ext4 -q $BOOT_PART >/dev/null 2>&1
    else
        mkfs.vfat -F32 $BOOT_PART >/dev/null 2>&1
    fi

    infobox "$_PrepParts" "\nCreating a $size ext4 root partition.\n" 0
    parted -s $device mkpart primary ext4 513MiB 100% >/dev/null 2>&1
    sleep 0.1
    ROOT_PART=$(lsblk -lno NAME,TYPE $device | awk 'NR == 3 {print "/dev/"$1}')
    mkfs.ext4 -q $ROOT_PART >/dev/null 2>&1

    tput civis; sleep 0.5
    msgbox "$_PrepParts" "\nAuto partitioning complete.\n\n$(lsblk $device -o NAME,MODEL,TYPE,FSTYPE,SIZE)"
}

mount_partition() {
    local part="$1"
    local mountp="${MNT}$2"
    local fs="$(lsblk -lno FSTYPE $part)"

    mkdir -p "$mountp"

    if [[ ${FS_OPTS[$fs]} != "" && $part != "$BOOT_PART" ]] && select_mount_opts "$part" "$fs"; then
        mount -o $MNT_OPTS $part "$mountp" 2>$ERR
        echeck "mount -o $MNT_OPTS $part $mountp"
    else
        mount $part "$mountp" 2>$ERR
        echeck "mount $part $mountp"
    fi

    confirm_mount $part "$mountp" || return 1
    check_cryptlvm "$part"
    return 0
}

find_partitions() {
    local str="$1"
    local err=''

    # string of partitions as /TYPE/PART SIZE
    if [[ $IGNORE_DEV != "" ]]; then
        PARTS="$(lsblk -lno TYPE,NAME,SIZE |
            awk "/$str/"' && !'"/$IGNORE_DEV/"' {sub(/^part/, "/dev/");
                sub(/^lvm|^crypt/, "/dev/mapper/"); print $1$2 " " $3}')"
    else
        PARTS="$(lsblk -lno TYPE,NAME,SIZE |
            awk "/$str/"' {sub(/^part/, "/dev/") sub(/^lvm|^crypt/, "/dev/mapper/")
                print $1$2 " " $3}')"
    fi

    # number of partitions total
    COUNT=$(wc -l <<< "$PARTS")

    # ensure we have enough partitions for the system and action type
    case $str in
        'part|lvm|crypt') [[ $COUNT -eq 0 || ($SYS == 'UEFI' && $COUNT -lt 2) ]] && err="$_PartErrBody" ;;
        'part|crypt') (( COUNT == 0 )) && err="$_LvmPartErrBody" ;;
        'part|lvm') (( COUNT < 2 )) && err="$_LuksPartErrBody" ;;
    esac

    # if there aren't enough partitions show the error message
    if [[ $err ]]; then
        msgbox "$_ErrTitle" "$err"
        return 1
    fi
    return 0
}

setup_boot_device() {
    [[ $BOOT_PART = /dev/nvme* ]] && BOOT_DEVICE="${BOOT_PART%p[1-9]}" ||
        BOOT_DEVICE="${BOOT_PART%[1-9]}"
    BOOT_PART_NUM="${BOOT_PART: -1}"

    parted -s $BOOT_DEVICE set $BOOT_PART_NUM boot on >/dev/null 2>&1
    if [[ $SYS == 'UEFI' ]]; then
        parted -s $BOOT_DEVICE set $BOOT_PART_NUM esp on >/dev/null 2>&1
    fi
    return 0
}

mountp() {
    # prepare partition list PARTS for dialog
    lvm_detect
    unmountp
    find_partitions 'part|lvm|crypt' || return 1

    # remove boot partition from dialog list if we auto partitioned one
    [[ $BOOT_PART != "" ]] && decr_count "$BOOT_PART"

    select_root_partition || return 1

    if [[ $BOOT_PART == "" ]]; then
        if [[ $SYS == "UEFI" ]]; then
            select_efi_partition || { BOOT_PART=""; return 1; }
        elif (( $COUNT > 0 )); then
            select_boot_partition || { BOOT_PART=""; return 1; }
        fi
    else
        infobox "$_PrepMount" "\nUsing boot partition: $BOOT_PART\n" 1
    fi

    select_boot_setup || { BOOTLDR=""; return 1; }

    if [[ $BOOT_PART != "" ]]; then
        infobox "$_PrepMount" "\nSetting up for the bootloader and mounting the boot partition.\n" 1
        setup_boot_device
        mount_partition "$BOOT_PART" "${BMNTS[$SYS-$BOOTLDR]}" || return 1
        SEPERATE_BOOT=true
    fi

    select_swap || return 1
    select_extra_partitions || return 1
    return 0
}

unmountp() {
    swapoff -a
    for i in $(mount | awk "/${MNT//\//\\/}/"' {print $3}' | sort -r); do
        umount "$i" >/dev/null 2>&1
    done
}

select_swap() {
    # Ask user to select partition or create swapfile
    tput civis
    SWAP="$(dialog --backtitle "$BT" --cr-wrap --stdout --title " $_SelSwpSetup " \
        --menu "$_SelSwpBody" 0 0 0 "$_SelSwpNone" "-" "$_SelSwpFile" "$SYS_MEM" $PARTS)"
    [[ $? != 0 || $SWAP == "$_SelSwpNone" ]] && { SWAP=""; return 0; }

    if [[ $SWAP == "$_SelSwpFile" ]]; then
        tput cnorm
        SWAP_SIZE="$(getinput "$_SelSwpSetup" "$_SelSwpSize" "$SYS_MEM")"
        [[ $? != 0 || $SWAP_SIZE == "" ]] && { SWAP=""; return 0; }

        while ! [[ ${SWAP_SIZE:0:1} =~ [1-9] && ${SWAP_SIZE: -1} =~ (M|G) ]]; do
            msgbox "$_SelSwpSetup Error" "\n$_SelSwpErr $SWAP_SIZE\n"
            SWAP_SIZE="$(getinput "$_SelSwpSetup" "$_SelSwpSize" "$SYS_MEM")"
            [[ $? != 0 || $SWAP_SIZE == "" ]] && { SWAP=""; break; return 0; }
        done

        enable_swap "$MNT/swapfile"
        SWAP="/swapfile"
    else
        enable_swap "$SWAP"
        decr_count "$SWAP"
    fi
    return 0
}

select_mountpoint() {
    tput cnorm
    EXTRA_MNT="$(getinput "$_PrepMount $part" "$_ExtPartBody1 /home /var\n" "/")"
    [[ $? != 0 || $EXTRA_MNT == "" ]] && return 1

    # bad mountpoint
    if [[ ${EXTRA_MNT:0:1} != "/" || ${#EXTRA_MNT} -le 1 || $EXTRA_MNT =~ \ |\' ]]; then
        msgbox "$_ErrTitle" "$_ExtErrBody"
        select_mountpoint || return 1
    fi
    return 0
}

select_boot_setup() {
    # choose bootloader and mountpoint (if needed)
    tput civis
    BOOTLDR="$(dialog --cr-wrap --stdout --backtitle "$BT" \
        --title " $_PrepMount " --menu "$_MntBootBody" 0 0 0 ${BOOTLDRS[$SYS]})"
    [[ $? != 0 || $BOOTLDR == "" ]] && return 1

    # grub on BIOS needs an install device, NOT partition eg. /dev/sda
    if [[ $SYS == 'BIOS' && $BOOTLDR == 'grub' && $BOOT_DEVICE == "" ]]; then
        select_device 'boot' || return 1
    fi

    if [[ $BOOTLDR == 'systemd-boot' ]]; then
        FILES[9]="/boot/loader/entries/$DIST.conf"
    elif [[ $BOOTLDR == 'syslinux' ]]; then
        if [[ $SYS == 'BIOS' ]]; then
            FILES[9]="/boot/syslinux/syslinux.cfg"
            BCMDS[$BOOTLDR]="$(dialog --cr-wrap --stdout --backtitle "$BT" \
                --title " $_InstSysTitle " --menu "$_InstSysBody" 0 0 0 \
                "syslinux-install_update -iam" "Install to MBR (Master Boot Record)" \
                "syslinux-install_update -i" "Install to root partition (/)")"
            [[ $? != 0 || ${BCMDS[$BOOTLDR]} == "" ]] && return 1
        else
            FILES[9]="/boot/EFI/syslinux/syslinux.cfg"
            BCMDS[syslinux]="efibootmgr -c -d $BOOT_DEVICE -p $BOOT_PART_NUM -l /EFI/syslinux/syslinux.efi -L $DIST"
        fi
    else
        FILES[9]="/etc/default/grub"
        if [[ $SYS == 'UEFI' ]]; then
            local ttype="i386-efi"
            [[ $IS_64BIT == true ]] && ttype='x86_64-efi'
            BCMDS[grub]="${BCMDS[grub]} --target=$ttype --bootloader-id=$DIST"
        else
            BCMDS[grub]="${BCMDS[grub]} --target=i386-pc $BOOT_DEVICE"
        fi

        BCMDS[grub]="${BCMDS[grub]} && grub-mkconfig -o /boot/grub/grub.cfg"
    fi
    return 0
}

select_mount_opts() {
    local part="$1"
    local fs="$2"
    local title="${fs^} Mount Options"

    tput civis
    if ssd "$part"; then
        MNT_OPTS="$(dialog --cr-wrap --stdout --backtitle "$BT" --title " $title " --checklist \
            "$_MntBody" 0 0 0 $(sed 's/discard - off/discard - on/' <<< "${FS_OPTS[$fs]}") | sed 's/ /,/g; $s/,$//')"
    else
        MNT_OPTS="$(dialog --cr-wrap --stdout --backtitle "$BT" --title " $title " \
            --checklist "$_MntBody" 0 0 0 ${FS_OPTS[$fs]} | sed 's/ /,/g; $s/,$//')"
    fi
    [[ $? != 0 || $MNT_OPTS == "" ]] && return 1

    if ! yesno "$title" "$_MntConfBody $MNT_OPTS\n"; then
        select_mount_opts "$part" "$fs" || return 1
    fi
    return 0
}

select_filesystem() {
    local part="$1"
    local cur_fs
    cur_fs="$(lsblk -lno FSTYPE $part)"
    tput civis

    local fs
    fs="$(dialog --cr-wrap --stdout --backtitle "$BT" --title " $_FSTitle: $part " \
        --menu "\nPartition Name: ${part}$([[ $cur_fs ]] &&
        printf "\nExisting Filesystem: %s" "$cur_fs")$_FSBody" 0 0 0 \
        $([[ $cur_fs != "" ]] && printf "%s -" "$_Skip") \
        "ext4" "${FS_CMDS[ext4]}"  "ext3" "${FS_CMDS[ext3]}" \
        "ext2" "${FS_CMDS[ext2]}"  "vfat" "${FS_CMDS[vfat]}" \
        "ntfs" "${FS_CMDS[ntfs]}"  "f2fs" "${FS_CMDS[f2fs]}" \
        "jfs" "${FS_CMDS[jfs]}"    "nilfs2" "${FS_CMDS[nilfs2]}" \
        "reiserfs" "${FS_CMDS[reiserfs]}" "xfs" "${FS_CMDS[xfs]}")"
    [[ $fs == "$_Skip" ]] && return 0
    [[ $fs == "" ]] && return 1

    if yesno "$_FSTitle" "\nFormat $part as $fs?\n"; then
        format $part $fs
    else
        select_filesystem $part || return 1
    fi
    return 0
}

select_efi_partition() {
    tput civis
    if (( COUNT == 1 )); then
        BOOT_PART="$(awk 'NF > 0 {print $1}' <<< "$PARTS")"
        infobox "$_PrepMount" "$_OnlyOne for EFI: $BOOT_PART\n" 1
    else
        BOOT_PART="$(dialog --cr-wrap --stdout --backtitle "$BT" \
            --title " $_PrepMount " --menu "$_SelUefiBody" 0 0 0 $PARTS)"
        [[ $? != 0 || $BOOT_PART == "" ]] && return 1
    fi

    if grep -q 'fat' <<< "$(fsck -N "$BOOT_PART")"; then
        local msg="$_FormUefiBody $BOOT_PART $_FormUefiBody2"
        yesno "$_PrepMount" "$msg" "Format $BOOT_PART" "Do Not Format" "no" &&
            format "$BOOT_PART" "vfat"
    else
        format "$BOOT_PART" "vfat"
    fi
    return 0
}

select_boot_partition() {
    tput civis
    BOOT_PART="$(dialog --cr-wrap --stdout --backtitle "$BT" \
        --title " $_PrepMount " --menu "$_SelBiosBody" 0 0 0 "$_Skip" "-" $PARTS)"

    if [[ $BOOT_PART == "$_Skip" || $BOOT_PART == "" ]]; then
        BOOT_PART=""
    else
        if grep -q 'ext[34]' <<< "$(fsck -N "$BOOT_PART")"; then
            local msg="$_FormBiosBody $BOOT_PART $_FormUefiBody2"
            yesno "$_PrepMount" "$msg" "Format $BOOT_PART" "Skip Formatting" "no" &&
                format "$BOOT_PART" "ext4"
        else
            format "$BOOT_PART" "ext4"
        fi
    fi
    return 0
}

select_root_partition() {
    # if we used LUKS and no LVM or LUKS+LVM remove the relevant partition labels from the list
    if [[ $LUKS && ! $LVM ]]; then
        ROOT_PART="/dev/mapper/$LUKS_NAME"
        decr_count "$LUKS_PART"
    elif [[ $LVM ]]; then
        [[ $LUKS ]] && decr_count "$LUKS_PART"
        for part in $(echo "$GROUP_PARTS"); do decr_count "$part"; done
        ROOT_PART=""
    fi

    if [[ $COUNT -eq 1 && ! $ROOT_PART ]]; then
        ROOT_PART="$(awk 'NF > 0 {print $1}' <<< "$PARTS")"
        infobox "$_PrepMount" "$_OnlyOne for root (/): $ROOT_PART\n" 1
    elif [[ ! $ROOT_PART || $LVM ]]; then
        tput civis
        ROOT_PART="$(dialog --cr-wrap --stdout --backtitle "$BT" \
            --title " $_PrepMount " --menu "$_SelRootBody" 0 0 0 $PARTS)"
        [[ $? != 0 || $ROOT_PART == "" ]] && return 1
    else
        local msg="\nUsing${LUKS} root partition:"  # $LUKS might just be an empty string
        infobox "$_PrepMount" "$msg $ROOT_PART\n" 1
    fi

    select_filesystem "$ROOT_PART" || { ROOT_PART=""; return 1; }
    mount_partition "$ROOT_PART" || { ROOT_PART=""; return 1; }
    return 0
}

select_extra_partitions() {
    while (( COUNT > 0 )); do
        tput civis
        local part
        part="$(dialog --cr-wrap --stdout --backtitle "$BT" \
            --title " $_PrepMount " --menu "$_ExtPartBody" 0 0 0 "$_Done" "-" $PARTS)"
        [[ $? != 0 || $part == "$_Done" || $part == "" ]] && break

        # choose what filesystem and get the mountpoint
        select_filesystem "$part" || { break; return 1; }
        select_mountpoint || { break; return 1; }

        # mount it
        mount_partition "$part" "$EXTRA_MNT" || { break; return 1; }
        EXTRA_MNTS="$EXTRA_MNTS $part: $EXTRA_MNT"

        # if the mountpoint was /usr add 'usr' to MKINIT_HOOKS
        [[ $EXTRA_MNT == "/usr" && $MKINIT_HOOKS != *usr* ]] && MKINIT_HOOKS="usr $MKINIT_HOOKS"
    done
    return 0
}
